package com.huawei.dli.flink.demo.sink;

import java.nio.charset.Charset;
import java.sql.PreparedStatement;

import com.huawei.hwclouds.drs.avro.Record;
import org.apache.flink.configuration.Configuration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HiveDsrSink extends HiveSink<Record> {
  private static final Logger logger = LoggerFactory.getLogger(HiveDsrSink.class);

  public HiveDsrSink(String principalName, String zkServerPrincipal, String dbName) {
    super(principalName, zkServerPrincipal, dbName);
  }

  @Override
  public void open(Configuration parameters) throws Exception {
    super.open(parameters);
  }

  @Override
  public void invoke(Record element, Context context) throws Exception {
    String operation = element.getOperation().toString().toUpperCase();
    String querySql = "";
    switch (operation) {
      case "INSERT":
        querySql = genInsertSql(element);
        break;
      case "DELETE":
        querySql = genDeleteSql(element);
        break;
      case "UPDATE":
        querySql = genUpdateSql(element);
        break;
      default:
        logger.warn("Unknown operation: " + operation + ", ignore it.");
    }
    PreparedStatement statement = null;
    try {
      statement = conn.prepareStatement(querySql);
      statement.executeUpdate();
    } catch (Exception ex) {
      logger.error("Failed to process sql: " + querySql);
    } finally {
      if (statement != null) {
        statement.close();
      }
    }
  }

  private String genInsertSql(Record element) {
    StringBuilder sql = new StringBuilder();
    StringBuilder values = new StringBuilder();
    sql.append("INSERT INTO ").append(element.getTableName().toString()).append("(");
    for (int i = 0; i < element.getFields().size(); i++) {
      sql.append(element.getFields().get(i).getName().toString()).append(",");

      // TODO: here only handle string value, the other value should be process accordingly.
      com.huawei.hwclouds.drs.avro.Character value =
        (com.huawei.hwclouds.drs.avro.Character) element.getAfterImages().get(i);
      Charset charset = Charset.forName(value.getCharset().toString());
      String strValue = new String(value.getValue().array(), charset);
      values.append(strValue).append(",");
    }
    sql.deleteCharAt(sql.length() - 1);
    values.deleteCharAt(values.length() - 1);
    return sql.append(") VALUES(").append(values.toString()).append(")").toString();
  }

  private String genDeleteSql(Record element) {
    // TODO: gen delete sql
    return "";
  }

  private String genUpdateSql(Record element) {
    // TODO: gen update sql
    return "";
  }
}
