package com.huawei.dli.flink.demo.sink;

import java.math.BigDecimal;
import java.sql.PreparedStatement;

import org.apache.flink.configuration.Configuration;
import org.apache.flink.types.Row;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Demo with json schema:
 * {"type":"object","properties":{"id":{"type":"string"},"name":{"type":"string"},"age":{"type":"integer"}}}
 */
public class HiveRowSink extends HiveSink<Row> {
  private static final Logger logger = LoggerFactory.getLogger(HiveRowSink.class);

  private transient PreparedStatement statement;

  public HiveRowSink(String principalName, String zkServerPrincipal, String dbName) {
    super(principalName, zkServerPrincipal, dbName);
  }

  @Override
  public void open(Configuration parameters) throws Exception {
    super.open(parameters);

    // TODO: use batch insert with: insert into xx values(), values()
    String querySql = "insert into testTb values(?, ?, ?)";
    statement = conn.prepareStatement(querySql);
  }

  @Override
  public void invoke(Row element, Context context) throws Exception {
    logger.info("id: " + element.getField(0));
    logger.info("name: " + element.getField(1));
    logger.info("age: " + element.getField(2));

    statement.setString(1, (String) element.getField(0));
    statement.setString(2, (String) element.getField(1));
    statement.setInt(3, ((BigDecimal) element.getField(2)).intValue());
    statement.executeUpdate();
  }

  @Override
  public void close() throws Exception {
    if (statement != null) {
      statement.close();
    }
    super.close();
  }
}
