package com.huawei.dli.flink.demo.sink;

import java.util.Optional;
import java.util.Properties;

import com.huawei.dli.flink.demo.utils.LoginUtil;
import org.apache.flink.api.common.serialization.SerializationSchema;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaException;
import org.apache.flink.streaming.connectors.kafka.FlinkKafkaProducer;
import org.apache.flink.streaming.connectors.kafka.partitioner.FlinkKafkaPartitioner;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class KafkaSink<T> extends FlinkKafkaProducer<T> {
  private static final Logger logger = LoggerFactory.getLogger(KafkaSink.class);

  private String tmpDir = "/tmp/kafkaSink.";
  private String principalName;

  public KafkaSink(String topic, SerializationSchema<T> serializationSchema,
      Properties properties, String userPrincipal) {
    super(topic, serializationSchema, properties);
    this.principalName = userPrincipal;

  }
  public KafkaSink(String topic, SerializationSchema<T> serializationSchema,
      Optional<FlinkKafkaPartitioner<T>> partitioner, Properties properties, String userPrincipal) {
    super(topic, serializationSchema, properties, partitioner);
    this.principalName = userPrincipal;
  }

  @Override
  public void open(Configuration parameters) throws Exception {
    securityPrepare();
    super.open(parameters);
  }

  @Override
  public void close() throws FlinkKafkaException {
    super.close();
    try {
      LoginUtil.deleteFile(tmpDir, "user.keytab");
      LoginUtil.deleteFile(tmpDir, "krb5.conf");
    } catch (Exception e) {
      logger.error("Failed to close source.", e);
    }
  }

  private void securityPrepare() throws Exception {
    String userKeytabFile = LoginUtil.createFile(tmpDir, "user.keytab", this.getClass());
    String krb5File = LoginUtil.createFile(tmpDir, "krb5.conf", this.getClass());
    LoginUtil.setKrb5Config(krb5File);
    LoginUtil.setJaasConf(LoginUtil.Module.KAFKA.getName(), principalName, userKeytabFile);
    sun.security.krb5.Config.refresh();
  }
}
