package com.huawei.dli.demo

import com.redislabs.provider.redis._
import org.apache.spark.rdd.RDD
import org.apache.spark.sql.{SaveMode, SparkSession}
import org.apache.spark.{SparkConf, SparkContext}

object SparkRedisDemoRdd {
  case class Person(name: String, age: Int)

  def main(args: Array[String]): Unit = {
    val spark = SparkSession
      .builder()
      .appName("datasource_redis")
      .config("spark.redis.host", "192.168.0.201")
      .config("spark.redis.port", "6379")
      .config("spark.redis.auth", "xxxx")
      .getOrCreate()

    val sc = spark.sparkContext

    // *********** read or write the redis with RDD *******************
    // Save Hash type data
    val hashRedisData:RDD[(String,String)] = sc.parallelize(
      Seq[(String,String)](("high", "111"), ("together", "333")))
    sc.toRedisHASH(hashRedisData, "hashRDD")

    // Read data via redis cli:
    //    HGETALL hashRDD
    //    high
    //    111
    //    together
    //    333

    // Save List type data
    val data = List(("school", "112"), ("tom", "333"))
    val listRedisData:RDD[String] = sc.parallelize(Seq[String](data.toString()))
    sc.toRedisLIST(listRedisData, "listRDD")

    // Save ZSet type data
    val zsetRedisData:RDD[(String, String)] = sc.parallelize(
      Seq[(String, String)](("xxx", "234"), ("dddd", "343")))
    sc.toRedisZSET(zsetRedisData, "zsetRDD")

    // Read all Hash type data
    sc.fromRedisHash("*").collect().foreach{println}
    // Read Hash type data by key
    sc.fromRedisHash(Array("hashRDD")).collect().foreach{println}

    // Read List type data
    sc.fromRedisList("*").collect().foreach{println}
    // Read List type data by key
    sc.fromRedisList(Array("listRDD")).collect().foreach{println}

    // Read ZSet type data
    sc.fromRedisZSetWithScore("*").collect().foreach{println}
    // Read Zset type data by key
    sc.fromRedisZSetWithScore(Array("zsetRDD")).collect().foreach{println}
    // Read Zset type data by range
    sc.fromRedisZRange(Array("zsetRDD"), 1, 3).collect().foreach{println}

    spark.stop()
  }
}
